%% FiPIt: A Simple, Fast Global Method for Solving Models with Two
%% Endogenous States & Occasionally Binding Constraints
%% Mendoza & Villalvazo 2020, Review of Economic Dynamics
% Main Code file
% (C.) Enrique G. Mendoza & Sergio Villalvazo, 2019
%% CELL 1. SET PARAMETER VALUES AND STATE SPACE
clear all
close all
clc

% Set the path of the MFiles folder
tMyDir      = pwd;
tIdcs       = strfind(tMyDir,'\');
tMyDirUp    = tMyDir(1:tIdcs(end)-1);
tMFilesPath = [tMyDirUp, '\MFiles\'];
addpath(tMFilesPath);

%parpool(3); %Use with Parallel toolbox when desired no. of parallel pool workers 
             %differs from no. specified in preferences or profile. Setting
             %to fewer than number of CPUs in the computer is faster.
             %If using "fzero" nonlinear solver, turning off parallel computing is
             %faster, if using "fsolve" solver, turning it on is faster. When
             %using it, change the loops marked as "this loop can be
             %parallelized" from "for" to "parfor". 

tic(); %start internal timer

% Model Parameters (taken from Mendoza AER (2010), except pBeta)
pSigma     = 2.0; % risk aversion CRRA utility function
pOmega     = 1.8461; % labor elasticity
pBeta      = 0.92; % discount factor
pAdjC      = 2.75; % capital adjustment cost
pPhi       = 0.2579; % working capital parameter
pDelta     = 0.08800443; % depreciation rate
pAlpha     = 0.5924043; % labor share
pEta       = 0.102229; % imported inputs share
pGamma     = 1 - pAlpha - pEta; % capital share of income, Cobb Douglas prod. function
pTaxCons   = 0.168615385890; % Tax on consumption
nProd      = 6.982; % Productivity parameter y
pDebtLimit = -300; % Ad-hoc debt limit for the RBC model
pBondMinSS = -200; % Minimum debt state such that SS model is feasible
pKappa     =0.2; %1.1; %0.2; %10.20; % limit on ratio of total debt tovalue of capital
% IMPORTANT: FOR SOLVING RBC MODEL, SET pKappa HIGH ENOUGH SO THAT CREDIT 
% CONSTRAINT NEVER BINDS. FOR SOLVING SS MODEL SET TO A LOW VALUE SUCH THAT 
% IT BINDS AT LEAST IN SOME STATES.
% With the above calibration pkappa>1 (<1) yields RBC (SS) solution

% Algorithm Parameters
pFixPointPriceK = 1; % Sets method to solve for q: 1=FiPIT method, 0=iterate to convergence on forward solution

nTol      = 0.0001; % 0.0001 epsilon^f convergence criterion for b', q and mu~
nTolInner = 0.0001; % convergence criterion for solving q (relevant only if q is solved with forward soln., pFixPointPriceK=0)
nTolDist  = 0.0000001; % convergence criterion for ergodic distribution
nIterTol  = 1000; % maximum number of iterations on b', q, mu~
nUpdateGuessB  = 1.00; % \rho^b for updating conjecture of b' and mu~
nUpdateGuessPK = 0.30; % \rho^k=\rho^mu for updating conjectures of q
% if needed, \rho^k, \rho^mu could be set to different values with proper
% editing

% Markov process of shocks (from Mendoza AER (2010):
% order of states: (Rl,pl,el), (Rl,pl,eh), (Rl,ph,el), (Rl,ph,eh), (Rh,pl,el), (Rh,pl,eh), (Rh,ph,el), (Rh,ph,eh)
% Transition probability matrix
mTran =[0.51363285,	0.16145114,	0.07773228,	0.02443373,	0.16145114,	0.03201937,	0.02443373,	0.00484575;...
        0.03201937,	0.64306463,	0.00484575,	0.09732026,	0.16145114,	0.03201937,	0.02443373,	0.00484575;...
        0.07773228,	0.02443373,	0.51363285,	0.16145114,	0.02443373,	0.00484575,	0.16145114,	0.03201937;...
        0.00484575,	0.09732026,	0.03201937,	0.64306463,	0.02443373,	0.00484575,	0.16145114,	0.03201937;...
        0.03201937,	0.16145114,	0.00484575,	0.02443373,	0.64306463,	0.03201937,	0.09732026,	0.00484575;...
        0.03201937,	0.16145114,	0.00484575,	0.02443373,	0.16145114,	0.51363285,	0.02443373,	0.07773228;...
        0.00484575,	0.02443373,	0.03201937,	0.16145114,	0.09732026,	0.00484575,	0.64306463,	0.03201937;...
        0.00484575,	0.02443373,	0.03201937,	0.16145114,	0.02443373,	0.07773228,	0.16145114,	0.51363285];
% Values of the stochastic process
vProdGrid    = [0.986595988257;	1.013404011625;	0.986595988257;	1.013404011625;	0.986595988257;	1.013404011625;	0.986595988257;	1.013404011625];
vIntRateGrid = [1.064449652804;	1.064449652804;	1.064449652804;	1.064449652804;	1.106970405389;	1.106970405389;	1.106970405389;	1.106970405389];
vPriceGrid   = [0.993455002946;	0.993455002946;	1.062225686321;	1.062225686321;	0.993455002946;	0.993455002946;	1.062225686321;	1.062225686321];
vIntRateGrid = vIntRateGrid - 1;
nShockGrid = size(mTran,1); % size of the vectorized stochastic process

% Bonds Grid (vBondGrid)
nBondGrid = 80; % number of nodes
nBondMin  = pDebtLimit; % value of lower bound
nBondMax  = 800 ; % value of upper bound
vBondGrid = zeros(nBondGrid,1); % initialize bonds grid
vBondGrid(1) = nBondMin;
for j = 2:nBondGrid
    vBondGrid(j) = vBondGrid(j-1)+(nBondMax-nBondMin)/((nBondGrid-1)); % fill bonds grid with evenly spaced values
end

% For SS solution, adjust bonds grid so it only has feasible states. After solving, 
% check the ergodic distribution of bonds to make sure pBondMinSS is not binding
% (i.e. endogenous credit constraint binds before ad-hoc debt limit).
if pKappa < 1  %Assumes <1 implies solving SS model
    pBondMinSSID = find(vBondGrid >= pBondMinSS,1,'first');
    vBondGrid    = vBondGrid(pBondMinSSID:end); % if solving SS model then bonds grid starts at pBondMinSS
    nBondGrid    = size(vBondGrid,1);
    nBondMin     = vBondGrid(1);
end

% Capital Grid (vCapitalGrid)
nCapitalCenter   = 770; % center value of the capital grid 
nCapitalGrid = 30; % number of nodes
nCapitalMin  = nCapitalCenter*0.85;  % lower bound, set in percent of center value
nCapitalMax  = nCapitalCenter*1.15 ; % upper bound, set in percent of center value
vCapitalGrid = zeros(nCapitalGrid,1); % initialize capitals grid
vCapitalGrid(1) = nCapitalMin;
for j = 2:nCapitalGrid
    vCapitalGrid(j) = vCapitalGrid(j-1)+(nCapitalMax-nCapitalMin)/((nCapitalGrid-1)); % fill capitals grid with evenly spaced values
end

%% CELL 2. INITIAL CONJECTURES OF b'(b,k,s), q(b,k,s) AND mu~(b,k,s) 
% WHERE s = (eps_y, r, p_m), INITIALIZATION OF ARRAYS, & NONLINEAR SOLVER OPTIONS

% Step 1) Initial conjectures for b',q,mu~
mBondPrimeGuess    = repmat(vBondGrid, 1, nCapitalGrid, nShockGrid); % Initial guess b'(b,k,s)=b
mPriceKGuess       = repmat(1, nBondGrid, nCapitalGrid, nShockGrid); % Initial guess q(b,k,s)=1, implies % Initial guess k'(b,k,s)=k
mMuHatGuess        = repmat(0, nBondGrid, nCapitalGrid, nShockGrid); % Initial guess mu~(b,k,s)=0
% Array definitions for other variables:
mPriceK            = mPriceKGuess;
mPriceKnew         = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mPriceKInner       = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mCapital           = repmat(vCapitalGrid', nBondGrid, 1, nShockGrid);
mBond              = repmat(vBondGrid, 1, nCapitalGrid, nShockGrid);
mDividend          = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mImplicitKappa     = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mCapitalPrimeGuess = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mInvGuess          = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mVGuess            = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mLGuess            = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mGDPGuess          = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mCGuess            = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mV                 = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mL                 = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mGDP               = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mC                 = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mBondPrime         = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mProdGrid          = permute(repmat(vProdGrid,1,nCapitalGrid,nBondGrid),[3 2 1]);
mPriceGrid         = permute(repmat(vPriceGrid,1,nCapitalGrid,nBondGrid),[3 2 1]);
mIntRateGrid       = permute(repmat(vIntRateGrid,1,nCapitalGrid,nBondGrid),[3 2 1]);

%

% Optimization options for nonlinear solver. "optimoptions(.)" for fsolve and
% "optimset(.)" for fzero 
pSolverOpt = optimset('Display','off'); %optimoptions('fsolve','Display','off','TolFun',1e-18); 
% The solver is used later to find allocations in states in which mu~>0 (relevant only if
% solving model with binding constraint and if binding constraint requires nonlinear solver). 
nMaxDif    = 10; %initialize convergence metric (some positive number to force some iterations to be executed)
nIter      = 0; % initialize iteration counter
fprintf('It    MaxDiff \n');
%    
%% CELL 3. MAIN LOOP ITERATING ON RECURSIVE FUNCTIONS FOR b',q,mu~
%   
% (nIter=iteration no. ("j" in Appendix), nMaxDif=value of convergence metric at iteration nIter)
while nMaxDif > nTol && nIter < nIterTol  
    mIdBind = ones(nBondGrid, nCapitalGrid, nShockGrid); %initializing variable for Step 3, indicator of coordinates where constraint binds
    mMuBond = repmat(0, nBondGrid, nCapitalGrid, nShockGrid); %initializing variable for Step 3, multiplier for b' hitting lower bound of B grid
%% Step 2) Compute implied decision rules for RBC and SS cases in iteration nIter
    % Implied investment and capital decision rules
    % (independent of mu~)
    mCapitalPrimeGuess = mCapital./pAdjC.*(mPriceKGuess - 1 + pAdjC);
    mInvGuess = mCapitalPrimeGuess - (1-pDelta)*mCapital + ...
                       pAdjC./(2.*mCapital).*(mCapitalPrimeGuess - mCapital).^2; 
                   
   % Implied factor allocations, output and consumption (depend on mu~ if mu~>0)   
    mVGuess = (mProdGrid.*nProd.*mCapital.^pGamma.*...
        pEta.^((pOmega-pAlpha)./pOmega).*(pAlpha./(1+pTaxCons)).^(pAlpha./pOmega) ./ (mPriceGrid.^((pOmega-pAlpha)./pOmega).*...
        (1+pPhi.*(mIntRateGrid) + mMuHatGuess.*pPhi.*(mIntRateGrid+1))) ).^...
        (pOmega./(pOmega.*(1-pEta) - pAlpha)) ; 
    mLGuess = (pAlpha./(pEta.*(1+pTaxCons)).*mPriceGrid.*mVGuess ).^(1/pOmega);
    mGDPGuess = nProd.*mProdGrid.*mCapital.^pGamma.*...
       mLGuess.^pAlpha.*mVGuess.^pEta - mPriceGrid.*mVGuess;
    mCGuess = (mGDPGuess -...
        pPhi.*(mIntRateGrid).*((1+pTaxCons).*mLGuess.^pOmega + mPriceGrid.*mVGuess) - ...
        mInvGuess - mBondPrimeGuess./(mIntRateGrid+1) + mBond)./(1+pTaxCons);               
                    
    
   %% Step 3) Solutions assuming collateral constraint does not bind 
    mMuHat = mMuHatGuess*0;  
    
    %% Step 3.1) factor allocations and gross output (easy, because they depend on K,TFP,R and p only)
    mV = (mProdGrid.*nProd.*mCapital.^pGamma.*...
        pEta.^((pOmega-pAlpha)./pOmega).*(pAlpha./(1+pTaxCons)).^(pAlpha./pOmega) ./ (mPriceGrid.^((pOmega-pAlpha)./pOmega).* ...
        (1+pPhi.*(mIntRateGrid) + mMuHat.*pPhi.*(mIntRateGrid+1))) ).^ ...
        (pOmega./(pOmega.*(1-pEta) - pAlpha)) ; 
    mL = (pAlpha./(pEta.*(1+pTaxCons)).*mPriceGrid.*mV).^(1./pOmega);
    mGDP = nProd.*mProdGrid.*mCapital.^pGamma.* ...
       mL.^pAlpha.*mV.^pEta - mPriceGrid.*mV; 
                   
      for iBond = 1:nBondGrid %this loop can be parallelized (change for to parfor)
        for iCapital = 1:nCapitalGrid
            for iShock = 1:nShockGrid
           %% Steps 3.2) and 3.3) Solve for new decision rules for consumption and bonds, and implied leverage ratio
                % fFiPIt_Cons.m solves for new consumption decision rule
                    mC(iBond,iCapital,iShock) = fFiPIt_Cons(mBondPrimeGuess(iBond,iCapital,iShock),pTaxCons,pEta,pAdjC,pPhi, pDelta, pBeta,pSigma,mTran,pAlpha,pGamma,pOmega,nProd,iBond,iCapital,iShock, nShockGrid,vPriceGrid,vProdGrid,vBondGrid,vCapitalGrid,vIntRateGrid,mCapitalPrimeGuess,mBondPrimeGuess,mMuHatGuess,mCGuess,mLGuess,mL);
                  %bonds decision rule follows from resource constraint  
                    mBondPrime(iBond,iCapital,iShock) = (vIntRateGrid(iShock)+1)*(mGDP(iBond,iCapital,iShock) -...
                    pPhi*(vIntRateGrid(iShock))*((1+pTaxCons)*mL(iBond,iCapital,iShock)^pOmega + vPriceGrid(iShock)*mV(iBond,iCapital,iShock)) - ...
                    mInvGuess(iBond,iCapital,iShock) + vBondGrid(iBond) - mC(iBond,iCapital,iShock)*(1+pTaxCons) );
                    %calculate implied leverage ratio:           
                mImplicitKappa(iBond,iCapital,iShock) = (-mBondPrime(iBond,iCapital,iShock)/(vIntRateGrid(iShock)+1) + pPhi*(vIntRateGrid(iShock)+1)*((1+pTaxCons)*mL(iBond,iCapital,iShock)^pOmega + vPriceGrid(iShock)*mV(iBond,iCapital,iShock)) ) / ...
                    (mPriceKGuess(iBond,iCapital,iShock)*mCapitalPrimeGuess(iBond,iCapital,iShock));
                
           %% Step 3.4) Check if the collateral constraint binds with the new decision rules
                if mBondPrime(iBond,iCapital,iShock)/(vIntRateGrid(iShock)+1) - pPhi*(vIntRateGrid(iShock)+1)*((1+pTaxCons)*mL(iBond,iCapital,iShock)^pOmega + vPriceGrid(iShock)*mV(iBond,iCapital,iShock)) + ...
                        pKappa*mPriceKGuess(iBond,iCapital,iShock)*mCapitalPrimeGuess(iBond,iCapital,iShock) > 0.0 % 0.000001 could use number slighltly ><0 for tolerance with which states are defined as nonbinding
                    mIdBind(iBond,iCapital,iShock) = 0;
                end 
                
                % Check if Bond decision is below ad-hoc debt limit (lower bound of B grid), and if
                % it does impose that the decision rule equals that debt
                % limit, adjust the consumption decision rule and evaluate
                % the multiplier associated with the ad-hoc debt limit.
                if mBondPrime(iBond,iCapital,iShock) <= nBondMin
                    mBondPrime(iBond,iCapital,iShock) = nBondMin;
                    
                    mC(iBond,iCapital,iShock) = (1/((1+pTaxCons)))*(mGDP(iBond,iCapital,iShock) -...
                    pPhi*(vIntRateGrid(iShock))*((1+pTaxCons)*mL(iBond,iCapital,iShock)^pOmega + vPriceGrid(iShock)*mV(iBond,iCapital,iShock)) - ...
                    mInvGuess(iBond,iCapital,iShock) + vBondGrid(iBond) - mBondPrime(iBond,iCapital,iShock)/(vIntRateGrid(iShock)+1) );
                
                    mMuBond(iBond,iCapital,iShock) = fFiPIt_MuBond(pTaxCons,pEta,pAdjC,pPhi, pDelta, pBeta,pSigma,mTran,pAlpha,pGamma,pOmega,nProd,iBond,iCapital,iShock, nShockGrid,vPriceGrid,vProdGrid,vBondGrid,vCapitalGrid,vIntRateGrid,mCapitalPrimeGuess,mBondPrimeGuess,mMuHatGuess,mCGuess,mLGuess);

                end

            end
        end
    end
    
    %% Step 4) Re-solve decision rules for states (b,k,s) when the constraint binds (and only when solving SS model)
    if pKappa < 1
        for iBond = 1:nBondGrid %this loop can be parallelized (change for to parfor)
            for iCapital = 1:nCapitalGrid
                for iShock = 1:nShockGrid
                    if mIdBind(iBond,iCapital,iShock) == 1 % Enters rest of this loop only if constraint binds
                  % fFiPIt_MuHat.m forms the non-linear equation (eq. (48) in the paper) needed to
                  % determine new value of mu~_j+1
                        fAux = @(x) fFiPIt_MuHat(x,mPriceKGuess,mCGuess,mLGuess,pKappa,pTaxCons,pEta,pAdjC,pPhi, pDelta, pBeta,pSigma,mTran,pAlpha,pGamma,pOmega,nProd,iBond,iCapital,iShock, nShockGrid,vPriceGrid,vProdGrid,vBondGrid,vCapitalGrid,vIntRateGrid,mCapitalPrimeGuess,mBondPrimeGuess,mMuHatGuess);
                        vInitX = mMuHatGuess(iBond,iCapital,iShock); %initial condition set to conjecture for mu~ at start of iteration
                        %Line below applies nonlinear solver. Can use "fsolve" or "fzero". 
                        %fzero is faster if there is only 1 equation and the function crosses the
                        %horizontal axis. Both use initial conditions set in vInitX and fFiPIt_MuHat function
                        [vSolTemp, res1] =fzero(fAux,vInitX,pSolverOpt); %fsolve(fAux,vInitX,pSolverOpt); 
                        mMuHat(iBond,iCapital,iShock) = vSolTemp(1);
                        %Now that mu~_j+1 is solved for, compute the factor
                        %allocations, output, bonds decision rule and consumption 
                        mV(iBond,iCapital,iShock) = (vProdGrid(iShock)*nProd*vCapitalGrid(iCapital)^pGamma*...
                            pEta^((pOmega-pAlpha)/pOmega)*(pAlpha/(1+pTaxCons))^(pAlpha/pOmega) / (vPriceGrid(iShock)^((pOmega-pAlpha)/pOmega)*...
                            (1+pPhi*(vIntRateGrid(iShock)) + mMuHat(iBond,iCapital,iShock)*pPhi*(vIntRateGrid(iShock)+1))) )^...
                            (pOmega/(pOmega*(1-pEta) - pAlpha)) ; 
                        mL(iBond,iCapital,iShock) = (pAlpha/(pEta*(1+pTaxCons))*vPriceGrid(iShock)*mV(iBond,iCapital,iShock) )^(1/pOmega);
                        mGDP(iBond,iCapital,iShock) = nProd*vProdGrid(iShock)*vCapitalGrid(iCapital)^pGamma*...
                           mL(iBond,iCapital,iShock)^pAlpha*mV(iBond,iCapital,iShock)^pEta - vPriceGrid(iShock)*mV(iBond,iCapital,iShock);

                        mBondPrime(iBond,iCapital,iShock) = (1+vIntRateGrid(iShock))*(-pKappa*mPriceKGuess(iBond,iCapital,iShock)*mCapitalPrimeGuess(iBond,iCapital,iShock) +...
                           pPhi*(vIntRateGrid(iShock)+1)*((1+pTaxCons)*mL(iBond,iCapital,iShock)^pOmega  + vPriceGrid(iShock)*mV(iBond,iCapital,iShock))) ;

                        mC(iBond,iCapital,iShock) = (mGDP(iBond,iCapital,iShock) -...
                            pPhi*(vIntRateGrid(iShock))*((1+pTaxCons)*mL(iBond,iCapital,iShock)^pOmega + vPriceGrid(iShock)*mV(iBond,iCapital,iShock)) - ...
                            mInvGuess(iBond,iCapital,iShock) - mBondPrime(iBond,iCapital,iShock)/(vIntRateGrid(iShock)+1) + vBondGrid(iBond))/(1+pTaxCons);  

                    end 
                end
            end
        end
    end
    %% Step 5) New decision rules for the conjectured b',mu~,q have been solved for all (b,k,s)
    
    %% Step 6) Now solve for the new pricing function
    % (includes alternatives using FiPIt (if pFixPointPriceK was set=1 in Cell 1)
    %  and using forward solution for q using k Euler eq (if pFixPointPriceK was set=0 in Cell 1)
    mPriceKGuessInner = mPriceKGuess;
    nMaxDifInner      = 10; %initialize q function convergence metric (some positive number to force some iterations to be executed)
    nIterInner        = 0;  % initialize iteration counter for iterations on q function
    while nMaxDifInner > nTolInner
          for iBond = 1:nBondGrid %this loop can be parallelized (change for to parfor)
            for iCapital = 1:nCapitalGrid
                for iShock = 1:nShockGrid
                    % fFiPIt_PriceK calculates eq. (51) in the paper
                    mPriceKInner(iBond,iCapital,iShock) = fFiPIt_PriceK(pKappa,pTaxCons,pEta,pAdjC,pPhi, pDelta, pBeta,pSigma,mTran,pAlpha,pGamma,pOmega,nProd,iBond,iCapital,iShock, nShockGrid,vPriceGrid,vProdGrid,vBondGrid,vCapitalGrid,vIntRateGrid,mCapitalPrimeGuess,mBondPrime,mPriceKGuessInner,mMuHat,mC,mL,mV);
                end
            end
        end
        
        nIterInner = nIterInner + 1; 
        
        %% Steps 6.1) and 6.2) 
        if pFixPointPriceK == 1 || nIter < 10  % step 6.1) uses FiPIt to generate new q value
            nMaxDifInner = 0;  %forces q convergence metric to zero to rule out iterations, since FiPIt does not need them 
            mPriceKGuessInner = mPriceKInner;
        else % step 6.2) iterates to convergence on forward solution for q from k Euler eq.
            nMaxDifInner = max(max(max(abs(mPriceKInner - mPriceKGuessInner)))) ; %updates q convergence metric when forward q solution is used
            mPriceKGuessInner = mPriceKInner;
        end 
    end
    mPriceK = mPriceKInner;
    
    %% Step 7) Check convergence of b'(.),mu~(.),q(.) and update conjectures
    nIter   = nIter + 1;   % update iteration counter
    nMaxDif = max( [max(max(max(abs(mBondPrime - mBondPrimeGuess)))) , ...
                    max(max(max(abs(mPriceK - mPriceKGuess)))) , ...
                    max(max(max(abs(mMuHat - mMuHatGuess)))) ] )  ; % compute value of convergence metric
    
    mBondPrimeGuess = (1-nUpdateGuessB)*mBondPrimeGuess + nUpdateGuessB*mBondPrime; %update b' conjecture
    mPriceKGuess    = (1-nUpdateGuessPK)*mPriceKGuess + nUpdateGuessPK*mPriceK; %update q conjecture
    mMuHatGuess     = (1-nUpdateGuessB)*mMuHatGuess + nUpdateGuessB*mMuHat; %update mu~ conjecture
    % note that b' and mu~ use the same updating weights, but these can be
    % edited to be different
  
    if (mod(nIter, 20) == 0 || nMaxDif < nTol) % prints to screen convergence metric every 20 iterations
        fprintf('%d    %f \n',nIter, nMaxDif);  
    end     
    
end
nTimeSeconds = toc(); %records elapsed time from internal timer
fprintf('Exec Time %f seconds \n',nTimeSeconds);

if pKappa < 1 %save output files with RBC or SS names as needed
    save solFiPIt_SS.mat 
else
    save solFiPIt_RBC.mat
end


%% CELL 4. GET EULER ERRORS

% The last guess already converged so it is the solution
mBondPrime = mBondPrimeGuess; 
mPriceK    = mPriceKGuess; 
mMuHat     = mMuHatGuess;
    
% Create Matrices
mC       = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mInv     = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mNX      = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mNX_GDP  = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mB_GDP   = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mMu      = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mW       = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mWC      = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mLev     = zeros(nBondGrid, nCapitalGrid, nShockGrid);

% Tobin Q equation
mCapitalPrime = mCapital./pAdjC.*(mPriceKGuess - 1 + pAdjC);

% Equilibrium recursive functions of all variables based on converged
% solutions for q,b',mu~
mV = (mProdGrid.*nProd.*mCapital.^pGamma.*pEta.^((pOmega-pAlpha)./pOmega).*...
    (pAlpha./(1+pTaxCons)).^(pAlpha./pOmega) ./ (mPriceGrid.^((pOmega-pAlpha)./pOmega).*...
        (1+pPhi.*(mIntRateGrid) + mMuHat.*pPhi.*(mIntRateGrid+1))) ).^...
        (pOmega./(pOmega.*(1-pEta) - pAlpha)) ; 

mL = (pAlpha./(pEta.*(1+pTaxCons)).*mPriceGrid.*mV ).^(1/pOmega);
            
mGDP = nProd.*mProdGrid.*mCapital.^pGamma.*...
   mL.^pAlpha.*mV.^pEta - mPriceGrid.*mV;

mInv = mCapitalPrime - (1-pDelta).*mCapital + pAdjC./2.*(mCapitalPrime - mCapital).^2./mCapital ;

mC = (mGDP - pPhi.*(mIntRateGrid).*((1+pTaxCons).*mL.^pOmega + mPriceGrid.*mV) - ...
    mInv - mBondPrimeGuess./(mIntRateGrid+1) + mBond)./(1+pTaxCons);  
            
mW = mL.^(pOmega-1)*(1+pTaxCons);

mWC = pPhi.*(mW.*mL + mPriceGrid.*mV);

mNX = mBondPrimeGuess./(mIntRateGrid + 1) - mBond + mIntRateGrid.*mWC;

mNX_GDP = mNX./mGDP;

mB_GDP = mBond./mGDP;

mMu = mMuHatGuess.*(mC - mL.^pOmega./pOmega).^-pSigma / (1+pTaxCons);

mLev = (mBondPrimeGuess./(1+mIntRateGrid) -...
    pPhi.*(1+mIntRateGrid).*(mW.*mL + mPriceGrid.*mV))./(mPriceKGuess.*mCapitalPrime);

mDividend = pGamma.*mProdGrid.*nProd.*mCapitalPrime.^(pGamma-1).*mL.^pAlpha.*mV.^pEta...
                - pDelta + pAdjC./2.*(mCapitalPrime-mCapital).^2./(mCapital.^2);
            
% Get Euler Errors
mEEbond    = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mEEcapital = zeros(nBondGrid, nCapitalGrid, nShockGrid);
  for iBond = 1:nBondGrid %this loop can be parallelized (change for to parfor)
    for iCapital = 1:nCapitalGrid
        for iShock = 1:nShockGrid
            
            x = [mBondPrime(iBond, iCapital, iShock), mCapitalPrime(iBond, iCapital, iShock)];
            iEEtemp = fFiPIt_EulerError(x,pKappa,pTaxCons,pEta,pAdjC,pPhi, pDelta, pBeta,pSigma,mTran,pAlpha,pGamma,pOmega,nProd,iBond,iCapital,iShock, nShockGrid,vPriceGrid,vProdGrid,vBondGrid,vCapitalGrid,vIntRateGrid,mCapitalPrime,mBondPrime,mMuHat,mMuHat(iBond,iCapital,iShock),mMuBond(iBond,iCapital,iShock),mC,mL,mV,mPriceKGuess) ;
            mEEbond(iBond,iCapital,iShock)    = iEEtemp(1) ;
            mEEcapital(iBond,iCapital,iShock) = iEEtemp(2);
            
         end
    end
end
% Get max Euler error value and position for the bond equation
nMaxEEbond = max(max(max((mEEbond)))); 
[nMaxEEbondPosB,nMaxEEbondPosC,nMaxEEbondPosS] = ind2sub(size(mEEbond),find(mEEbond == nMaxEEbond));
if isempty(nMaxEEbondPosB) == 1
    [nMaxEEbondPosB,nMaxEEbondPosC,nMaxEEbondPosS] = ind2sub(size(mEEbond),find(mEEbond == -nMaxEEbond));
end
% Get max Euler error value and position for the capital equation
nMaxEEcapital = max(max(max((mEEcapital))));
[nMaxEEcapitalPosB,nMaxEEcapitalPosC,nMaxEEcapitalPosS] = ind2sub(size(mEEcapital),find(mEEcapital == nMaxEEcapital));
if isempty(nMaxEEcapitalPosB) == 1
    [nMaxEEcapitalPosB,nMaxEEcapitalPosC,nMaxEEcapitalPosS] = ind2sub(size(mEEcapital),find(mEEcapital == -nMaxEEcapital));
end
% Get average Euler Errors
nMeanEEbond    = mean(mean(mean((mEEbond)))) ;
nMeanEEcapital = mean(mean(mean((mEEcapital))));

if pKappa < 1
    save solFiPIt_SS.mat 
else
    save solFiPIt_RBC.mat
end

%% CELL 5 GET THE ERGODIC DISTRIBUTION 

% Create Matrices for the Ergodic Distribution
mErgDist      = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mErgDistGuess = ones(nBondGrid, nCapitalGrid, nShockGrid).*(1/(nBondGrid*nCapitalGrid*nShockGrid));

mOutGrid         = zeros(nBondGrid, nCapitalGrid, nShockGrid);
 
mUpperBondPos    = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mLowerBondPos    = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mUpperCapitalPos = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mLowerCapitalPos = zeros(nBondGrid, nCapitalGrid, nShockGrid);

mUpperBond    = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mLowerBond    = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mUpperCapital = zeros(nBondGrid, nCapitalGrid, nShockGrid);
mLowerCapital = zeros(nBondGrid, nCapitalGrid, nShockGrid);

  for iBond = 1:nBondGrid %this loop can be parallelized (change for to parfor)
    for iCapital = 1:nCapitalGrid
        for iShock = 1:nShockGrid  
            % Limit the upper limit of the bond policy function to obtain
            % convergence in the erdogic distribution. At the end make
            % sure the upper limit is not binding
            mBondPrimeGuess(iBond,iCapital,iShock) = min(nBondMax,mBondPrimeGuess(iBond,iCapital,iShock));
            
            % Find the grid points such that:
            % ibL <= bp and ikL <= kp
            % if bp of kp is out of the grid use the closest grid points
            % to extrapolate
            ibL = find(mBondPrimeGuess(iBond,iCapital,iShock) >= vBondGrid,1,'last');
            ikL = find(mCapitalPrime(iBond,iCapital,iShock) >= vCapitalGrid,1,'last');         
            if isempty(ikL) == 1 
                ikL = 0;      
                mOutGrid(iBond,iCapital,iShock) = 1;
            end
            if isempty(ibL) == 1
                ibL = 0;  
                mOutGrid(iBond,iCapital,iShock) = 1;
            end
            if ikL == nCapitalGrid
                mOutGrid(iBond,iCapital,iShock) = 1;
            end
            if ibL == nBondGrid
                mOutGrid(iBond,iCapital,iShock) = 1;
            end
            mLowerBondPos(iBond,iCapital,iShock)    = ibL;
            mUpperBondPos(iBond,iCapital,iShock)    = ibL + 1;
            mLowerCapitalPos(iBond,iCapital,iShock) = ikL;
            mUpperCapitalPos(iBond,iCapital,iShock) = ikL + 1;
            
            if mOutGrid(iBond,iCapital,iShock) == 0
                mLowerBond(iBond,iCapital,iShock)    = vBondGrid(ibL);
                mUpperBond(iBond,iCapital,iShock)    = vBondGrid(ibL + 1);
                mLowerCapital(iBond,iCapital,iShock) = vCapitalGrid(ikL);
                mUpperCapital(iBond,iCapital,iShock) = vCapitalGrid(ikL + 1);
           end
                  
        end
    end
end

% Iterate until convergence of the Ergodic Distribution
nMaxDif3 = 10;
nIter3   = 0;
fprintf('It    MaxDiff Distribution \n');
while nMaxDif3 > nTolDist 
    
    mErgDist      = zeros(nBondGrid, nCapitalGrid, nShockGrid);
    % This for loop can't be parallelized since mErgDist is used both as an
    % input and output (it is a loop that needs sequential execution)
    for iBond = 1:nBondGrid
        for iCapital = 1:nCapitalGrid
            for iShock = 1:nShockGrid  
                for iShockP = 1:nShockGrid 

                    ibL = mLowerBondPos(iBond,iCapital,iShock);
                    ibU = mUpperBondPos(iBond,iCapital,iShock);
                    ikL = mLowerCapitalPos(iBond,iCapital,iShock);
                    ikU = mUpperCapitalPos(iBond,iCapital,iShock);
                    
                    % K' below grid and B' below grid
                    if ikL == 0 && ibL == 0
                        ikL = 1;
                        ibL = 1;
                        mErgDist(ibL, ikL, iShockP) = mErgDist(ibL, ikL, iShockP) + ...
                            mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                            ( (vBondGrid(ibU) - mBondPrimeGuess(iBond,iCapital,iShock))/(vBondGrid(ibU+1) - vBondGrid(ibL))*...
                             (vCapitalGrid(ikU) - mCapitalPrime(iBond,iCapital,iShock))/(vCapitalGrid(ikU+1) - vCapitalGrid(ikL)) );
                    % K' below grid and B' inside grid
                    elseif ikL == 0 && ibL > 0 && ibL < nBondGrid
                        ikL = 1;
                        mErgDist(ibL, ikL, iShockP) = mErgDist(ibL, ikL, iShockP) + ...
                            mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                            ( (vBondGrid(ibU) - mBondPrimeGuess(iBond,iCapital,iShock))/(vBondGrid(ibU) - vBondGrid(ibL))*...
                             (vCapitalGrid(ikU) - mCapitalPrime(iBond,iCapital,iShock))/(vCapitalGrid(ikU+1) - vCapitalGrid(ikL)) );
                         mErgDist(ibU, ikL, iShockP) = mErgDist(ibU, ikL, iShockP) + ...
                            mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                            ( (mBondPrimeGuess(iBond,iCapital,iShock) - vBondGrid(ibL))/(vBondGrid(ibU) - vBondGrid(ibL))*...
                             (vCapitalGrid(ikU) - mCapitalPrime(iBond,iCapital,iShock))/(vCapitalGrid(ikU+1) - vCapitalGrid(ikL)) );
             
                    % K' below grid and B' above grid
                    elseif ikL == 0 && ibL == nBondGrid
                        ikL = 1;
                        ibU = nBondGrid;
                         mErgDist(ibU, ikL, iShockP) = mErgDist(ibU, ikL, iShockP) + ...
                            mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                            ( (mBondPrimeGuess(iBond,iCapital,iShock) - vBondGrid(ibL))/(vBondGrid(ibU) - vBondGrid(ibL-1))*...
                             (vCapitalGrid(ikU) - mCapitalPrime(iBond,iCapital,iShock))/(vCapitalGrid(ikU+1) - vCapitalGrid(ikL)) );
             
                    % K' inside grid and B' below grid
                    elseif ikL > 0 && ikL < nCapitalGrid && ibL == 0
                        ibL = 1;
                        mErgDist(ibL, ikL, iShockP) = mErgDist(ibL, ikL, iShockP) + ...
                        mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                        ( (vBondGrid(ibU) - mBondPrimeGuess(iBond,iCapital,iShock))/(vBondGrid(ibU+1) - vBondGrid(ibL))*...
                         (vCapitalGrid(ikU) - mCapitalPrime(iBond,iCapital,iShock))/(vCapitalGrid(ikU) - vCapitalGrid(ikL)) );
                     mErgDist(ibL, ikU, iShockP) = mErgDist(ibL, ikU, iShockP) + ...
                        mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                        ( (vBondGrid(ibU) - mBondPrimeGuess(iBond,iCapital,iShock))/(vBondGrid(ibU+1) - vBondGrid(ibL))*...
                         (mCapitalPrime(iBond,iCapital,iShock) - vCapitalGrid(ikL))/(vCapitalGrid(ikU) - vCapitalGrid(ikL)) );
             
                             
                    % K' inside grid and B' inside grid
                    elseif ikL > 0 && ikL < nCapitalGrid && ibL > 0 && ibL < nBondGrid
                      mErgDist(ibL, ikL, iShockP) = mErgDist(ibL, ikL, iShockP) + ...
                        mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                        ( (vBondGrid(ibU) - mBondPrimeGuess(iBond,iCapital,iShock))/(vBondGrid(ibU) - vBondGrid(ibL))*...
                         (vCapitalGrid(ikU) - mCapitalPrime(iBond,iCapital,iShock))/(vCapitalGrid(ikU) - vCapitalGrid(ikL)) );
                    mErgDist(ibL, ikU, iShockP) = mErgDist(ibL, ikU, iShockP) + ...
                        mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                        ( (vBondGrid(ibU) - mBondPrimeGuess(iBond,iCapital,iShock))/(vBondGrid(ibU) - vBondGrid(ibL))*...
                         (mCapitalPrime(iBond,iCapital,iShock) - vCapitalGrid(ikL))/(vCapitalGrid(ikU) - vCapitalGrid(ikL)) );

                     mErgDist(ibU, ikL, iShockP) = mErgDist(ibU, ikL, iShockP) + ...
                        mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                        ( (mBondPrimeGuess(iBond,iCapital,iShock) - vBondGrid(ibL))/(vBondGrid(ibU) - vBondGrid(ibL))*...
                         (vCapitalGrid(ikU) - mCapitalPrime(iBond,iCapital,iShock))/(vCapitalGrid(ikU) - vCapitalGrid(ikL)) );
                    mErgDist(ibU, ikU, iShockP) = mErgDist(ibU, ikU, iShockP) + ...
                        mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                        ( (mBondPrimeGuess(iBond,iCapital,iShock) - vBondGrid(ibL))/(vBondGrid(ibU) - vBondGrid(ibL))*...
                         (mCapitalPrime(iBond,iCapital,iShock) - vCapitalGrid(ikL))/(vCapitalGrid(ikU) - vCapitalGrid(ikL)) );
             
                             
                    % K' inside grid and B' above grid
                    elseif ikL > 0 && ikL < nCapitalGrid && ibL == nBondGrid
                        ibU = nBondGrid;
                        mErgDist(ibU, ikL, iShockP) = mErgDist(ibU, ikL, iShockP) + ...
                            mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                            ( (mBondPrimeGuess(iBond,iCapital,iShock) - vBondGrid(ibL))/(vBondGrid(ibU) - vBondGrid(ibL-1))*...
                             (vCapitalGrid(ikU) - mCapitalPrime(iBond,iCapital,iShock))/(vCapitalGrid(ikU) - vCapitalGrid(ikL)) );
                         
                        mErgDist(ibU, ikU, iShockP) = mErgDist(ibU, ikU, iShockP) + ...
                            mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                            ( (mBondPrimeGuess(iBond,iCapital,iShock) - vBondGrid(ibL))/(vBondGrid(ibU) - vBondGrid(ibL-1))*...
                             (mCapitalPrime(iBond,iCapital,iShock) - vCapitalGrid(ikL))/(vCapitalGrid(ikU) - vCapitalGrid(ikL)) );
             
                             
                    % K' above grid and B' below grid
                    elseif ikL == nCapitalGrid && ibL == 0 
                        ikU = nCapitalGrid;
                        ibL = 1;
                        mErgDist(ibL, ikU, iShockP) = mErgDist(ibL, ikU, iShockP) + ...
                        mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                        ( (vBondGrid(ibU) - mBondPrimeGuess(iBond,iCapital,iShock))/(vBondGrid(ibU+1) - vBondGrid(ibL))*...
                         (mCapitalPrime(iBond,iCapital,iShock) - vCapitalGrid(ikL))/(vCapitalGrid(ikU) - vCapitalGrid(ikL-1)) );
                     
      
                             
                    % K' above grid and B' inside grid
                    elseif ikL == nCapitalGrid && ibL > 0 && ibL < nBondGrid
                        ikU = nCapitalGrid;
                        mErgDist(ibL, ikU, iShockP) = mErgDist(ibL, ikU, iShockP) + ...
                        mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                        ( (vBondGrid(ibU) - mBondPrimeGuess(iBond,iCapital,iShock))/(vBondGrid(ibU) - vBondGrid(ibL))*...
                         (mCapitalPrime(iBond,iCapital,iShock) - vCapitalGrid(ikL))/(vCapitalGrid(ikU) - vCapitalGrid(ikL-1)) );
                     
                         mErgDist(ibU, ikU, iShockP) = mErgDist(ibU, ikU, iShockP) + ...
                            mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                            ( (mBondPrimeGuess(iBond,iCapital,iShock) - vBondGrid(ibL))/(vBondGrid(ibU) - vBondGrid(ibL))*...
                             (mCapitalPrime(iBond,iCapital,iShock) - vCapitalGrid(ikL))/(vCapitalGrid(ikU) - vCapitalGrid(ikL-1)) );
             
                             
                    % K' above grid and B' above grid
                    elseif ikL == nCapitalGrid && ibL == nBondGrid
                        ikU = nCapitalGrid;
                        ibU = nBondGrid;                     
                         mErgDist(ibU, ikU, iShockP) = mErgDist(ibU, ikU, iShockP) + ...
                            mTran(iShock, iShockP)*mErgDistGuess(iBond,iCapital,iShock)*...
                            ( (mBondPrimeGuess(iBond,iCapital,iShock) - vBondGrid(ibL))/(vBondGrid(ibU) - vBondGrid(ibL-1))*...
                             (mCapitalPrime(iBond,iCapital,iShock) - vCapitalGrid(ikL))/(vCapitalGrid(ikU) - vCapitalGrid(ikL-1)) );
             
                             
                    end                                  
                                      
                end
            end
        end
    end
    
    nIter3 = nIter3 + 1;  
    % Get the max difference
    nMaxDif3 = max(max(max(abs(mErgDist - mErgDistGuess))));
    % Update the conjectured distribution
    mErgDistGuess  = mErgDist;
    
    if (mod(nIter3, 100) == 0 || nMaxDif3 < nTolDist) % prints to screen convergence metric every 100 iterations
        fprintf('%d    %.9f \n',nIter3, nMaxDif3);  
    end   
    
end
% Make sure the distribution sums to 1
mErgDist = mErgDist./sum(sum(sum(mErgDist)));

% Get non-binding Kappa that has positive probability
mImplicitKappaPosProb = zeros(nBondGrid, nCapitalGrid, nShockGrid);
  for iBond = 1:nBondGrid %this loop can be parallelized (change for to parfor)
    for iCapital = 1:nCapitalGrid
        for iShock = 1:nShockGrid 
            if mErgDist(iBond,iCapital,iShock) >= 0.0000001
                mImplicitKappaPosProb(iBond,iCapital,iShock) = mImplicitKappa(iBond,iCapital,iShock);
            end
        end
    end
end
nMinKappa = max(max(max(mImplicitKappaPosProb)));

% Save plots of the distribution
if pKappa >= 1 
    figure(1);clf;
    set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
    plot(vBondGrid,sum(sum(mErgDist,3),2),'LineWidth',3.0);
    axis([pDebtLimit nBondMax 0 0.12])
    %title('Ergodic Bond Distribution')
    set(gca,'FontSize',30)
    saveas(figure(1), 'ErgoBondDist_RBC.pdf');

    figure(1);clf;
    set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
    plot(vCapitalGrid,sum(sum(mErgDist,3),1),'LineWidth',3.0);
    %title('Ergodic Capital Distribution')
    axis([-inf inf 0 0.12])
    set(gca,'FontSize',30)
    saveas(figure(1), 'ErgoCapitalDist_RBC.pdf');
else
    figure(1);clf;
    set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
    plot(vBondGrid,sum(sum(mErgDist,3),2),'LineWidth',3.0);
    axis([pDebtLimit nBondMax 0 0.12])
    %title('Ergodic Bond Distribution')
    set(gca,'FontSize',30)
    saveas(figure(1), 'ErgoBondDist_SS.pdf');

    figure(1);clf;
    set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
    plot(vCapitalGrid,sum(sum(mErgDist,3),1),'LineWidth',3.0);
    axis([-inf inf 0 0.12])
    %title('Ergodic Capital Distribution')
    set(gca,'FontSize',30)
    saveas(figure(1), 'ErgoCapitalDist_SS.pdf');
end

if pKappa >= 1 
    figure(1);clf;
    set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
    surf(vBondGrid,vCapitalGrid,mErgDist(:,:,7)')
    axis([-300 400 600 900 0 0.0023])
    xlabel('Bond')
    ylabel('Capital')
    zlabel('Ergodic Distribution')
    set(gca,'FontSize',20)
    saveas(figure(1), 'ErgoDist_RBC.pdf');
else
    figure(1);clf;
    set(figure(1),'PaperType','usletter','PaperOrientation','Landscape','PaperPosition',[0.1 0.1 11 8.5]);
    surf(vBondGrid,vCapitalGrid,mErgDist(:,:,7)')
    axis([-300 400 600 900 0 0.0023])
    xlabel('Bond')
    ylabel('Capital')
    zlabel('Ergodic Distribution')
    set(gca,'FontSize',20)
    saveas(figure(1), 'ErgoDist_SS.pdf');
end


if pKappa < 1
    save solFiPIt_SS.mat 
else
    save solFiPIt_RBC.mat
end
