% 
%
% Function: fFiPIt_Cons
% Uses Fix Point Iteration to obtain consumption decision rule 
% (C.) Enrique G. Mendoza & Sergio Villalvazo, 2018
function [Y] = fFiPIt_Cons(x,pTaxCons,pEta,pAdjC,pPhi, pDelta, pBeta,pSigma,mTran,pAlpha,pGamma,pOmega,nProd,iBond,iCapital,iShock, nShockGrid,vPriceGrid,vProdGrid,vBondGrid,vCapitalGrid,vIntRateGrid,mCapitalPrimeGuess,mBondPrimeGuess,mMuHatGuess,mCGuess,mLGuess,mL)
        
        % Store the size of the bond and capital grids
        nB = size(vBondGrid,1);
        nK = size(vCapitalGrid,1);
        
        % Store the values that will be interpolated
        bp = x(1);
        kp = mCapitalPrimeGuess(iBond, iCapital, iShock);
        
        % Initialize the Expectation values
        EE1 = 0;
        
        % This FOR loop is to obtain the expected value
        for isp = 1:nShockGrid
            
            % Bilinear interpolation to get k'' (or kt+2) and b'' ( or bt+2)
            kX = kp;
            bX = bp;          
            
            % Locate the grid nodes below and above the values indicated
            % by the iteration-j bond and capital decision rules
            ikA = 0;
            iAbove = 0;
            while iAbove == 0 && ikA < nK
               ikA = ikA + 1;
               if kp < vCapitalGrid(ikA)
                   iAbove = 1;
               end
            end
            if iAbove == 1 && ikA > 1
               ikA = ikA - 1;
            elseif ikA == nK
               ikA = nK - 1;
            end

            ibA = 0;
            iAbove = 0;
            while iAbove == 0 && ibA < nB
               ibA = ibA + 1;
               if bp < vBondGrid(ibA)
                   iAbove = 1;
               end
            end
            if iAbove == 1 && ibA > 1
               ibA = ibA - 1;
            elseif ibA == nB
               ibA = nB - 1;
            end
            
            % Obtain the gridpoints such that kA <= kp <= kB
            kA = vCapitalGrid(ikA);
            kB = vCapitalGrid(ikA+1);
            % Obtain the gridpoints such that bA <= bp <= bC
            bA = vBondGrid(ibA);
            bC = vBondGrid(ibA+1);
            
            % Interpolate iteration-j consumption decision rule
            % to obtain its c' (or ct+1) value
            fA = mCGuess(ibA,ikA,isp);
            fB = mCGuess(ibA,ikA+1,isp);
            fC = mCGuess(ibA+1,ikA,isp);
            fD = mCGuess(ibA+1,ikA+1,isp);
            
            cp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
            
            % Interpolate itreation-j Labor decision rule
            % to obtain its  L' (or Lt+1) value
            fA = mLGuess(ibA,ikA,isp);
            fB = mLGuess(ibA,ikA+1,isp);
            fC = mLGuess(ibA+1,ikA,isp);
            fD = mLGuess(ibA+1,ikA+1,isp);
            
            Lp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
            % Interpolation ends
                        
            MgUp = (cp - (Lp^pOmega)/pOmega)^(-pSigma); %marginal utility at t+1 u'(t+1)
            EE1 = EE1 + mTran(iShock,isp)*MgUp;  %E[u'(t+1)]
            
         
        end
        
        % Call relevant values of labor and interest rate for Euler eq. 
        L = mL(iBond,iCapital,iShock); %value of labor at date t from j+1 labor decision rule
        R  = vIntRateGrid(iShock) + 1; %interest rate value
            
        % Solve for new decision rule for c (ct) using Euler Eq
        Y(1) = (R*pBeta*EE1)^(-1/pSigma) + (L^pOmega)/pOmega;
        
end
        
        
        
        