% 
%
% Function: fFiPIt_MuHat
% Forms the function passed on to non-linear solver to obtain multiplier MuHat 
% (C.) Enrique G. Mendoza & Sergio Villalvazo, 2018
function [Y] = fFiPIt_MuHat(x,mPriceKGuess,mCGuess,mLGuess,pKappa,pTaxCons,pEta,pAdjC,pPhi, pDelta, pBeta,pSigma,mTran,pAlpha,pGamma,pOmega,nProd,iBond,iCapital,iShock, nShockGrid,vPriceGrid,vProdGrid,vBondGrid,vCapitalGrid,vIntRateGrid,mCapitalPrimeGuess,mBondPrimeGuess,mMuHatGuess)
        
        % x(1) is the variable that will be solved by the non-linear solver
        MuHat = x(1);
        
        % Store the size of the bond and capital grids
        nB = size(vBondGrid,1);
        nK = size(vCapitalGrid,1);
        
        % Store the values that will be interpolated
        bp = mBondPrimeGuess(iBond, iCapital, iShock);
        kp = mCapitalPrimeGuess(iBond, iCapital, iShock);
        
        % Initialize the Expectation values
        EE1 = 0;
         
        % This FOR loop is to obtain the expected value
        for isp = 1:nShockGrid
            
            % Bilinear interpolation to get c' and L'
            kX = kp;
            bX = bp;   
            
            % Here we locate the grid points below and above the
            % conjectured policies
            ikA = 0;
            iAbove = 0;
            while iAbove == 0 && ikA < nK
               ikA = ikA + 1;
               if kp < vCapitalGrid(ikA)
                   iAbove = 1;
               end
            end
            if iAbove == 1 && ikA > 1
               ikA = ikA - 1;
            elseif ikA == nK
               ikA = nK - 1;
            end

            ibA = 0;
            iAbove = 0;
            while iAbove == 0 && ibA < nB
               ibA = ibA + 1;
               if bp < vBondGrid(ibA)
                   iAbove = 1;
               end
            end
            if iAbove == 1 && ibA > 1
               ibA = ibA - 1;
            elseif ibA == nB
               ibA = nB - 1;
            end
            
            % Obtain the gridpoints such that kA <= kp <= kB
            kA = vCapitalGrid(ikA);
            kB = vCapitalGrid(ikA+1);
            % Obtain the gridpoints such that bA <= bp <= bC
            bA = vBondGrid(ibA);
            bC = vBondGrid(ibA+1);
            
            % Interpolate c to obtain c'
            fA = mCGuess(ibA,ikA,isp);
            fB = mCGuess(ibA,ikA+1,isp);
            fC = mCGuess(ibA+1,ikA,isp);
            fD = mCGuess(ibA+1,ikA+1,isp);
            
            cp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
            
            % Interpolate L to obtain L'
            fA = mLGuess(ibA,ikA,isp);
            fB = mLGuess(ibA,ikA+1,isp);
            fC = mLGuess(ibA+1,ikA,isp);
            fD = mLGuess(ibA+1,ikA+1,isp);
            
            Lp = fBiLinearInterpolation(kA, kB, bA, bC, fA, fB, fC, fD, kX, bX);
            % Interpolation ends
            
            % Obtain the expected value
            
            MgUp = (cp - (Lp^pOmega)/pOmega)^(-pSigma);
            EE1 = EE1 + mTran(iShock,isp)*MgUp;  
                  
        end
        
        % Get variables needed in Equation (48)
        k = vCapitalGrid(iCapital);
        b = vBondGrid(iBond);
        
        qk = mPriceKGuess(iBond,iCapital,iShock);
        R = vIntRateGrid(iShock)+1;
        
        v = (vProdGrid(iShock)*nProd*k^pGamma*...
                    pEta^((pOmega-pAlpha)/pOmega)*(pAlpha/(1+pTaxCons))^(pAlpha/pOmega) / (vPriceGrid(iShock)^((pOmega-pAlpha)/pOmega)*...
                    (1+pPhi*(vIntRateGrid(iShock)) + MuHat*pPhi*(vIntRateGrid(iShock)+1))) )^...
                    (pOmega/(-pAlpha + pOmega*(1-pEta))) ; 
        L = (pAlpha/(pEta*(1+pTaxCons))*vPriceGrid(iShock)*v )^(1/pOmega);

        y = nProd*vProdGrid(iShock)*k^pGamma*L^pAlpha*v^pEta;
        
        inv = kp - (1-pDelta)*k + pAdjC/2*( kp-k )^2/k;
        
        
        C1 = (1/(1+pTaxCons))^(1-pEta)*nProd*vProdGrid(iShock)*k^pGamma*(pEta/(pAlpha*vPriceGrid(iShock)))^pEta ; 
        C2 = 1/pOmega + pEta/pAlpha + pPhi*(1 + pEta/pAlpha)*(2*R-1);
        
        % Form Equation (48)
        Y(1) = (1-MuHat)*(C1^(pOmega/((1-pEta)*pOmega-pAlpha)) *...
            (pAlpha/(1+pPhi*(R-1) + MuHat*pPhi*R))^((pEta*pOmega + pAlpha)/((1-pEta)*pOmega-pAlpha)) - ...
            (pAlpha*C1/(1+pPhi*(R-1) + MuHat*pPhi*R))^(pOmega/((1-pEta)*pOmega-pAlpha))*C2 - ...
            ((inv - pKappa*qk*kp-b)/(1+pTaxCons)) )^(-pSigma) - pBeta*R*EE1;
        
         
 
end